/**
 * PatternDemo.java
 *
 * The main program for an example demonstrating Observer and Strategy
 * design patterns.
 * 
 * @author  Jeff Ondich
 * @version 2.0.0
 * date:    2013.11.12
 */

import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;

public class PatternDemo extends JPanel
                         implements ChangeListener,
                                    ActionListener,
                                    PDSubject {

    private static final int SLIDER_MINIMUM = 0;
    private static final int SLIDER_MAXIMUM = 100;
    private static final int SLIDER_INITIAL_VALUE = 20;

    private ArrayList<PDObserver> observerList;

    private JSlider slider;
    private JButton bouncerButton;
    private JButton numberLabelButton;
    private JButton thermometerButton;

    /**
     * Sets up the demo panel.
     */
    public PatternDemo() {
        this.observerList = new ArrayList<PDObserver>();

        // Initialize the slider.
        this.slider = new JSlider(JSlider.HORIZONTAL, SLIDER_MINIMUM, SLIDER_MAXIMUM, SLIDER_INITIAL_VALUE);
        this.slider.addChangeListener(this);
        this.slider.setMajorTickSpacing(20);
        this.slider.setMinorTickSpacing(5);
        this.slider.setPaintTicks(true);
        this.slider.setPaintLabels(true);

        // Initialize the buttons.
        this.bouncerButton = new JButton("Bouncer");
        this.bouncerButton.setAlignmentX(Component.CENTER_ALIGNMENT);
        this.bouncerButton.setActionCommand("bouncer");
        this.bouncerButton.addActionListener(this);

        this.numberLabelButton = new JButton("Number Label");
        this.numberLabelButton.setAlignmentX(Component.CENTER_ALIGNMENT);
        this.numberLabelButton.setActionCommand("numberlabel");
        this.numberLabelButton.addActionListener(this);

        this.thermometerButton = new JButton("Thermometer");
        this.thermometerButton.setAlignmentX(Component.CENTER_ALIGNMENT);
        this.thermometerButton.setActionCommand("thermometer");
        this.thermometerButton.addActionListener(this);

        // Lay out the elements of this frame window.
        this.setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
        this.add(this.slider);
        this.add(this.bouncerButton);
        this.add(this.numberLabelButton);
        this.add(this.thermometerButton);
    }

    // PDSubject interfaces.
    public void subscribe(PDObserver observer) {
        if (this.observerList.indexOf(observer) < 0) {
            this.observerList.add(observer);
        }
    }

    public void unsubscribe(PDObserver observer) {
        this.observerList.remove(observer);
    }

    public void notifyObservers() {
        for (PDObserver observer : this.observerList) {
            observer.update(this);
        }
    }

    public int getValue() {
        return (int)this.slider.getValue();
    }

    // ChangeListener interface, for the slider.
    public void stateChanged(ChangeEvent e) {
        JSlider sourceSlider = (JSlider)e.getSource();
        assert sourceSlider == this.slider;
        this.notifyObservers();
    }

    // ActionListener interface, for the buttons.
    public void actionPerformed(ActionEvent e) {
        PDPanel panel = null;
        if ("bouncer".equals(e.getActionCommand())) {
            panel = new PDBouncerPanel();
        } else if ("thermometer".equals(e.getActionCommand())) {
            panel = new PDThermometerPanel();
        } else if ("numberlabel".equals(e.getActionCommand())) {
            panel = new PDNumberLabelPanel();
        } else {
            panel = new PDNumberLabelPanel();
            System.out.println("whoops");
        }

        // New frame with content panel
        JFrame frame = new JFrame(e.getActionCommand());
        final PDPanel finalPanel = panel;
        frame.addWindowListener( 
            new WindowAdapter() { 
                public void windowClosing(WindowEvent e) { 
                    //System.out.println(String.format("Unsubscribing. There are %d observers.", observerList.size()));
                    unsubscribe(finalPanel);
                    //System.out.println(String.format("Unsubscribed. There are %d observers.", observerList.size()));
                } 
            } 
        );

        frame.setSize(400, 300);
        JPanel contentPane = new JPanel();
        contentPane.setLayout(new BorderLayout());

        // Put a combo box in the frame
        String[] numberStrategies = {"Arabic", "Roman"};
        JComboBox comboBox = new JComboBox(numberStrategies);
        contentPane.add(comboBox, BorderLayout.PAGE_START);

        // Put the panel in the frame, and make it listen to the combo box
        this.subscribe(panel);
        comboBox.addActionListener(panel);
        panel.update(this);
        contentPane.add(panel, BorderLayout.CENTER);

        // Done.
        frame.setContentPane(contentPane);
        frame.pack();
        frame.setSize(300, 200);
        frame.setVisible(true);
    }

    // Main program
    public static void main(String[] args) {
        JFrame frame = new JFrame("PatternDemo");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setBackground(Color.white);

        PatternDemo patternDemoPanel = new PatternDemo();
        frame.add(patternDemoPanel, BorderLayout.CENTER);
        frame.pack();
        frame.setSize(400, 300);
        frame.setVisible(true);
    }
}

